classdef Problem15 < ImplicitGSIPProblem
    % p=Problem15 creates an instance of the GSIP problem:
    %
    %       min   4x1^2-x2-x2^2
    %
    %       s.t.  x2-y2 <=0 for all y in Y(x)
    %             -2+10^-6 <=x1 <= 2
    %             -3 <=x2 <= 2
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              (y1+y2)^2-y3<= 0
    %               y1 <=x1
    %               y2 <=x1    
    %              -4 <= y1 <= 4
    %              -4 <= y2 <= 4
    %              0  <= y3 <= 16}
        
    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=2;                          
        boundsOnUppLevVars=[-3,-3;2,2];             
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[0,0];                
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[-1 ,0, 2-1.e-6];          
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=3;                          
        nrOfLowLevNonlinConstrs=1;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=4*x(1)^2-x(2)-x(2)^2;
        end
        function grad_f=gradObjFun(obj,x)                    
            grad_f=[8*x(1);-1-2*x(2)];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)          
            g=x(2)-y(2);
            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)      
           
            n=size(y,1);
            o=ones(n,1);
            z=zeros(n,1);
            Dx_g=[z,o];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)            
            n=size(y,1);
            o=ones(n,1);
            z=zeros(n,1);
            Dy_g=[z,-o,z];                
        end
               
                
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)           
            u=[(y(1)+y(2))^2-y(3)];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)              
            Dy_u=[2*(y(1)+y(2)),2*(y(1)+y(2)),-1];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)              
            Dx_u=[0,0];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)           
            y1=(-2+x(1))/2;
            y2=(-4-y1+x(1))/2;
            y3=((y1+y2)^2+16)/2;
            y=[y1,y2,y3];                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            Dx_y=[1/2,0;1/4,3/2*(-2.5+3/4*x(1))];
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    %Schranken f�r die Variablen der unteren Stufe in Abh�ngigkeit von x
             lb=[-4,-4,0];
             ub=[4,4,16];
        end
        
        %transformation
        function y=evalTrafoFun(~,~,x,z) 
            y1=-4*z(:,1)+(1-z(:,1))*x(1);
            y2=max(-4,-4-y1).*z(:,2)+(1-z(:,2))*x(1);
            y3=z(:,3).*(y1+y2).^2+(1-z(:,3))*16;
            
            y=[y1,y2,y3];

        end
        
        function z=evalInvTrafoFun(~,~,x,y) 
            z1=(x(1)-y(:,1))/(x(1)+4);
            y2_l=max(-4,-4-y(:,1));
            z2=(x(1)-y(:,2))/(x(1)-y2_l);
           
            
            z3=(16-y(:,3))./(16-(y(:,1)+y(:,2)).^2);
            z=[z1,z2,z3];
            
        end
        
        function Dx_y=DxTrafoFun(~,~,x,z) 
  
            
            n=size(z,1);
            Dx_y=zeros(3,2,n);
            Dx_y(1,1,:)=1-z(:,1);
            y1=-4*z(:,1)+(1-z(:,1))*x(1);
            y2=max(-4,-4-y1).*z(:,2)+(1-z(:,2))*x(1);
            if-4>(-4-y1)
               Dx_y(2,1,:)=1-z(:,2);
            else
               Dx_y(2,1,:)=-z(:,2).*(Dx_y(1,1))+1-z(:,2);

            end
               temp=zeros(n,1);
               temp(:)=(Dx_y(1,1,:)+Dx_y(2,1,:));
               Dx_y(3,1,:)=2*z(:,3).*(y1+y2).*temp;
        end
        
        
        
    end
    
end
